package dev.kske.chess;

import java.awt.Color;
import java.awt.Container;
import java.awt.Dimension;
import java.awt.Graphics;
import java.awt.Image;
import java.awt.Point;
import java.awt.event.ComponentAdapter;
import java.awt.event.ComponentEvent;
import java.awt.event.MouseAdapter;
import java.awt.event.MouseEvent;
import java.io.File;
import java.util.HashMap;
import java.util.Map;

import javax.swing.JPanel;

import dev.kske.chess.piece.Piece;

/**
 * Project: <strong>Chess</strong><br>
 * File: <strong>BoardPanel.java</strong><br>
 * Created: <strong>01.07.2019</strong><br>
 * Author: <strong>Kai S. K. Engelbart</strong><br>
 * <br>
 * A square panel for rendering the chess board. To work correctly,
 * this must be added to a parent component that allows the child to decide the
 * size.
 */
public class BoardPanel extends JPanel {

	private static final long serialVersionUID = 6771148331334310216L;

	private static int tileSize;

	private static Map<String, Image> textures;

	private Board board;

	public BoardPanel(Board board) {
		this();
		setSize(getPreferredSize());
		setBoard(board);
	}

	public BoardPanel() {
		/*
		 * Add a component listener for adjusting the tile size on resizing.
		 * The size of the board is assumed to be 8x8, as well as the both the board and
		 * the tiles being square.
		 */
		addComponentListener(new ComponentAdapter() {

			@Override
			public void componentResized(ComponentEvent e) {
				tileSize = getWidth() / 8;

				// Load the piece textures if they are not present
				if (textures == null) loadPieceTextures();
			}
		});

		// Add a mouse adapter for testing piece movement
		addMouseListener(new MouseAdapter() {

			private Point src;

			@Override
			public void mousePressed(MouseEvent evt) {
				if (src == null) src = evt.getPoint();
				else {
					Point dest = evt.getPoint();
					board.move(src.x / tileSize, src.y / tileSize, dest.x / tileSize, dest.y / tileSize);
					repaint();
					src = null;
				}
			}
		});
	}

	@Override
	protected void paintComponent(Graphics g) {
		super.paintComponent(g);

		// Draw the board
		g.setColor(Color.white);
		for (int i = 0; i < 8; i++)
			for (int j = 0; j < 8; j++) {
				if (j > 0) g.setColor(g.getColor().equals(Color.white) ? Color.lightGray : Color.white);
				g.fillRect(tileSize * i, tileSize * j, tileSize, tileSize);
			}

		// Draw the pieces if a board is present
		if (board != null) for (int i = 0; i < 8; i++)
			for (int j = 0; j < 8; j++)
				if (board.getBoardArr()[i][j] != null)
					g.drawImage(getPieceTexture(board.getBoardArr()[i][j]), i * tileSize, j * tileSize, this);
	}

	/**
	 * Load every PNG file inside the res/pieces directory.
	 * The filenames without extensions are used as keys in the map textures.
	 */
	private static void loadPieceTextures() {
		textures = new HashMap<>();
		File	dir		= new File("res/pieces");
		File[]	files	= dir.listFiles((File parentDir, String name) -> name.toLowerCase().endsWith(".png"));
		for (File file : files)
			textures.put(file.getName().replaceFirst("[.][^.]+$", ""), TextureLoader.loadScaledImage(file, tileSize));
	}

	/**
	 * Loads a piece texture fitting to a piece object
	 * 
	 * @param piece The piece from which the texture properties are taken
	 * @return The fitting texture
	 */
	private static Image getPieceTexture(Piece piece) {
		String key = piece.getType().toString().toLowerCase() + "_" + piece.getColor().toString().toLowerCase();
		return textures.get(key);
	}

	@Override
	public Dimension getMinimumSize() { return getPreferredSize(); }

	@Override
	public Dimension getMaximumSize() { return getPreferredSize(); }

	@Override
	public Dimension getPreferredSize() {
		Dimension	d	= super.getPreferredSize();
		Container	c	= getParent();
		if (c != null) d = c.getSize();
		else return new Dimension(480, 480);
		int	w	= (int) d.getWidth();
		int	h	= (int) d.getHeight();
		int	s	= Math.max(w, h);
		return new Dimension(s, s);
	}

	public Board getBoard() { return board; }

	public void setBoard(Board board) { this.board = board; }
}
